# $1 = type
# $2 = variable name
# $3 = field name
# $4 = options
# $5 = value
BEGIN {
	FS="|"
	output=""
}

{ 
	valid_type = 0
	valid = 1
	# XXX: weird hack, but it works...
	n = split($0, param, "|")
	value = param[5]
	for (i = 6; i <= n; i++) value = value FS param[i]
	verr = ""
}

$1 == "int" {
	valid_type = 1
	if (value !~ /^[0-9]*$/) { valid = 0; verr = "Invalid value" }
}

$1 == "float" {
	valid_type = 1
	if (value !~ /^[0-9\.]*$/) { valid = 0; verr = "Invalid value" }
}

# FIXME: add proper netmask validation
($1 == "ip") || ($1 == "netmask") {
	valid_type = 1
	if ((value != "") && (value !~ /^[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*$/)) valid = 0
	else {
		split(value, ipaddr, "\\.")
		for (i = 1; i <= 4; i++) {
			if ((ipaddr[i] < 0) || (ipaddr[i] > 255)) valid = 0
		}
	}
	if (valid == 0) verr = "Invalid value"
}

$1 == "wep" {
	valid_type = 1
	if (value !~ /^[0-9A-Fa-f]*$/) {
		valid = 0
		verr = "Invalid value"
	} else if ((length(value) != 0) && (length(value) != 10) && (length(value) != 26)) {
		valid = 0
		verr = "Invalid key length"
	}
}

$1 == "wepascii" {
	valid_type = 1
	if ((length(value) != 0) && (length(value) != 5) && (length(value) != 13)) {
		valid = 0
		verr = "Invalid key length"
	}
}

$1 == "hostname" {
	valid_type = 1
	if ((value !~ /^[-0-9a-zA-Z]*$/) || (value ~ /[\\]/)) {
		valid = 0
		verr = "Invalid value. Can only contain letters, numbers, and -."
	}
}

$1 == "filedir" {
	valid_type = 1
	if (((value !~ /^[^"\/\\ ][^"\/\\]*[^"\/\\ ]$/) && (value !~ /^[^"\/\\ ]$/)) || (value ~ /[ ][ ]/)) {
		valid = 0
		verr = "Invalid value. File and Directory names can not include &quot;, /, &#92; or leading/trailing/double spaces."
	}
}

$1 == "ssid" {
	valid_type = 1
	if ((value !~ /^[\x20-\x7E]*$/) || (value ~ /[\\"]/)) {
		valid = 0
		verr = "Invalid value. SSIDs can only contain standard Alphanumeric and punctuation characters (Ascii 0x20-0x7F)."
	}
}

$1 == "password" {
	valid_type = 1
	if ((value !~ /^[\x20-\x7E]*$/) || (value ~ /[\\"]/)) {
		valid = 0
		verr = "Invalid value. Passwords can only contain standard Alphanumeric and punctuation characters (Ascii 0x20-0x7F)."
	}
}

$1 == "configval" {
	valid_type = 1
	if (value ~ /[\\"`$]/) {
		valid = 0
		verr = "Invalid value. Config strings can not contain &quot;, $, &#92 or &#96."
	}
}

$1 == "string" {
	valid_type = 1
}

$1 == "mac" {
	valid_type = 1
	if ((value != "") && (value !~ /^[0-9a-fA-F][0-9a-fA-F]:[0-9a-fA-F][0-9a-fA-F]:[0-9a-fA-F][0-9a-fA-F]:[0-9a-fA-F][0-9a-fA-F]:[0-9a-fA-F][0-9a-fA-F]:[0-9a-fA-F][0-9a-fA-F]$/)) {
		valid = 0
		verr = "Invalid value"
	}
}

$1 == "port" {
	valid_type = 1
	if (value !~ /^[0-9]*$/) {
		valid = 0
		verr = "Invalid value"
	}
}

$1 == "ports" {
	valid_type = 1
	n = split(value ",", ports, ",")
	for (i = 1; i <= n; i++) {
		if ((ports[i] !~ /^[0-9]*$/) && (ports[i] !~ /^[0-9][0-9]*-[0-9][0-9]*$/)) {
			valid = 0
			verr = "Invalid value"
		}
	}
}

$1 == "wpapsk" {
	valid_type = 1
	if (length(value) > 64) {
		valid = 0
		verr = "String too long"
	}
	if ((length(value) != 0) && (length(value) < 8)) {
		valid = 0
		verr = "String too short"
	}
	if ((length(value) == 64) && (value ~ /[^0-9a-fA-F]/)) {
		valid = 0
		verr = "Invalid hex key"
	}
}

valid_type != 1 { valid = 0 }

valid == 1 {
	n = split($4, options, " ")
	for (i = 1; (valid == 1) && (i <= n); i++) {
		if (options[i] == "required") {
			if (value == "") { valid = 0; verr = "No value entered" }
		} else if (options[i] == "cfg") {
			if (value ~ /[\\"`$]/) {
				valid = 0
				verr = "Invalid value. Config strings can not contain &quot;, $, &#92 or &#96."
			}
		} else if ((options[i] ~ /^min=/) && (value != "")) {
			min = options[i]
			sub(/^min=/, "", min)
			min = int(min)
			if ($1 == "int") {
				if (value < min) { valid = 0; verr = "Value too small (minimum: " min ")" }
			} else if ($1 == "string" || $1 == "hostname") {
				if (length(value) < min) { valid = 0; verr = "String too short (minimum length: " min ")"}
			} else if ($1 == "password") {
				if (length(value) < min) { valid = 0; verr = "Password too short (minimum length: " min ")"}
			}
		} else if ((options[i] ~ /^max=/) && (value != ""))  {
			max = options[i]
			sub(/^max=/, "", max)
			max = int(max)
			if ($1 == "int") {
				if (value > max) { valid = 0; verr = "Value too large (maximum: " max ")" }
			} else if ($1 == "string" || $1 == "hostname") {
				if (length(value) > max) { valid = 0; verr = "String too long (maximum length: " max ")" }
			} else if ($1 == "password") {
				if (length(value) > max) { valid = 0; verr = "Password too long (maximum length: " max ")" }
			}
		} else if ((options[i] == "nodots") && ($1 == "hostname")) {
			if (value ~ /\./) {
				valid = 0
				verr = "Invalid value"
			}
		}
	}
}

valid_type == 1 {
	if (valid == 1) output = output 
	else error = error "Error in " $3 ": " verr "<br />"
}

END {
	print output "ERROR=\"" error "\";\n"
	if (error == "") print "return 0"
	else print "return 255"
}
